package net.locusworks.discord.eighttrack.events.main.handlers;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.events.main.AbstractMainEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Component
public class NextSongHandler extends AbstractMainEventHandler {

  private ApplicationLogger logger;
  
  @Autowired
  private RepositoryService guildRepoService;
  
  public NextSongHandler() {
    super("-next");
    logger = ApplicationLoggerFactory.getLogger(NextSongHandler.class);
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
    try {
      musicHandler.next(event, commands);
    } catch (Exception ex) {
      logger.error("Unable to play next song: " + ex.getMessage(), ex);
      guildRepoService.logEntry(event.getGuild().getIdLong(), ex, "Unable to play next song: %s", ex.getMessage());
      sendMessage(event, "Sorry I am unable to play next song: " + ex.getMessage());
    }
  }
  
  @Override
  public String help() {
    return "-next <id of song to play> (will stop song if one already playing)";
  }

}
