package net.locusworks.discord.eighttrack.events.main.handlers;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.events.main.AbstractMainEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Component
public class PlaySongHandler extends AbstractMainEventHandler {

  private ApplicationLogger logger;
  
  @Autowired
  private RepositoryService guildRepoService;
  
  public PlaySongHandler() {
    super("-play");
    logger = ApplicationLoggerFactory.getLogger(PlaySongHandler.class);
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
    try {
      musicHandler.play(event, commands);
      musicHandler.isPlaying(true);
    } catch (Exception ex) {
      logger.error("Unable to play song: " + ex.getMessage(), ex);
      guildRepoService.logEntry(event.getGuild().getIdLong(), ex, "Unable to play song: %s", ex.getMessage());
      sendMessage(event, "Sorry I am unable to play requested song: " + ex.getMessage());
    }
  }
  
  @Override
  public String help() {
    return "-play <id of song to play> (will not play song if a song is already playing)";
  }

}
