package net.locusworks.discord.eighttrack.events.main.handlers;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.events.main.AbstractMainEventHandler;
import net.locusworks.discord.eighttrack.events.playlist.AbstractPlaylistEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Component
public class PlaylistHandler extends AbstractMainEventHandler {

  private ApplicationLogger logger;
  
  @Autowired
  private RepositoryService guildRepoService;

  private Map<String, AbstractPlaylistEventHandler> handlers;
  
  public PlaylistHandler() {
    super("-playlist");
    logger = ApplicationLoggerFactory.getLogger(PlaylistHandler.class);
  }
  
  @Autowired
  private void init(List<AbstractPlaylistEventHandler> eventHandlers) {
    handlers = new TreeMap<>();
    for(AbstractPlaylistEventHandler apeh : eventHandlers) {
      handlers.put(apeh.getCommand(), apeh);
    }
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
    if (commands == null || commands.isEmpty()) {
      event.getChannel().sendMessage("Missing command for -playlist. Valid commands are: add, delete, list, play").queue();
      return;
    }
    try {
      String command = commands.remove(0);
      
      AbstractPlaylistEventHandler apeh = handlers.get(command);
      if (apeh == null) {
        sendMessage(event, help());
        return;
      }
      
      apeh.executeEvent(musicHandler, event, commands);
      
    } catch (Exception ex) {
      logger.error("Unable to display playlist: " + ex.getMessage(), ex);
      guildRepoService.logEntry(event.getGuild().getIdLong(), ex, "Unable to display playlist: %s", ex.getMessage());
      sendMessage(event, "Sorry I am unable to display the playlist: " + ex.getMessage());
    }
  }
  
  @Override
  public String help() {
    
    StringBuilder sb = new StringBuilder();
    for(Entry<String, AbstractPlaylistEventHandler> entry : handlers.entrySet()) {
      sb.append(entry.getValue().help()).append("\n");
    }
    
    return "-playlist | " + sb.toString();
  }

}
