package net.locusworks.discord.eighttrack.events.main.handlers;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.events.main.AbstractMainEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Component
public class StopSongHandler extends AbstractMainEventHandler {

  private ApplicationLogger logger;
  
  @Autowired
  private RepositoryService guildRepoService;
  
  public StopSongHandler() {
    super("-stop");
    logger = ApplicationLoggerFactory.getLogger(StopSongHandler.class);
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
    try {
      musicHandler.isPlaying(false);
      musicHandler.stop(event);
      event.getGuild().getAudioManager().closeAudioConnection();
    } catch (Exception ex) {
      logger.error("Unable to stop song: " + ex.getMessage(), ex);
      guildRepoService.logEntry(event.getGuild().getIdLong(), ex, "Unable to stop song: %s", ex.getMessage());
      sendMessage(event, "Sorry I am unable to stop current song: " + ex.getMessage());
    }
  }
  
  @Override
  public String help() {
    return "-stop (stops the current song)";
  }

}
