package net.locusworks.discord.eighttrack.events.playlist.handlers;

import java.util.List;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.database.entities.GuildPlaylist;
import net.locusworks.discord.eighttrack.database.entities.GuildPlaylistSong;
import net.locusworks.discord.eighttrack.database.entities.GuildSong;
import net.locusworks.discord.eighttrack.events.playlist.AbstractPlaylistEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;

@Component
public class PlaylistListHandler extends AbstractPlaylistEventHandler {
  
  @Autowired
  private RepositoryService guildRepoService;

  public PlaylistListHandler() {
    super("list");
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
    if (commands == null || commands.isEmpty()) {
      List<GuildPlaylist> userPlaylist = guildRepoService.getGuildPlaylistRepo().findByGuildAndUserIdFetchSongs(event.getGuild().getIdLong(), event.getMember().getIdLong());
      if (userPlaylist.isEmpty()) {
        event.getChannel().sendMessage(event.getMember().getAsMention() + " you have no defined playlists on this server").queue();
        return;
      }

      int longestPlaylist = 10;
      for(GuildPlaylist gpl : userPlaylist) {
        if (gpl.getPlaylist().length() > longestPlaylist) longestPlaylist = gpl.getPlaylist().length();
      }

      String fmt = "%6s | %-" + longestPlaylist +"s | %s%n"; 

      StringBuilder sb = new StringBuilder("```");
      sb.append(String.format(fmt, "", "Playlist", "Song Count"));
      sb.append(StringUtils.repeat("-", 24 + longestPlaylist)).append("\n");

      int count = 0;
      for(GuildPlaylist gpl : userPlaylist) {
        sb.append(String.format(fmt, ++count, gpl.getPlaylist(), gpl.getGuildPlaylistSongList().size()));
      }
      sb.append("```");
      event.getChannel().sendMessage("**" + "Currently available playlists for " + event.getMember().getAsMention() + "**").queue((succcess)-> 
      event.getChannel().sendMessage(sb.toString()).queue());
      return;
    }

    String playlist = commands.remove(0);

    GuildPlaylist gpl = guildRepoService.getGuildPlaylistRepo().findByGuildUserPlaylistFetchSongs(event.getGuild().getIdLong(), event.getMember().getIdLong(), playlist);
    if (gpl == null) {
      event.getChannel().sendMessage(event.getMember().getAsMention() + " you have no defined playlists on this server by the name of " + playlist).queue();
      return;
    }

    if (gpl.getGuildPlaylistSongList().isEmpty()) {
      event.getChannel().sendMessage(event.getMember().getAsMention() + " you have no defined songs for playlist " + playlist).queue();
      return;
    }

    int longestSong = 0;
    int longestArtist = 0;
    for(GuildPlaylistSong gpls : gpl.getGuildPlaylistSongList()) {
      GuildSong gs = gpls.getGuildSong();
      if (gs.getSong().getTitle().length() > longestSong) longestSong = gs.getSong().getTitle().length();
      if (gs.getSong().getArtist().length() > longestArtist) longestArtist = gs.getSong().getArtist().length();
    }

    String fmt = "%6s | %-" + longestSong +"s | %-" + longestArtist +"s | %s%n"; 
    int count = 0;
    StringBuilder sb = new StringBuilder("```");
    sb.append(String.format(fmt, "Track", "Title", "Artist", "id"));
    sb.append(StringUtils.repeat("-", 27 + longestSong + longestArtist)).append("\n");

    for(GuildPlaylistSong gpls : gpl.getGuildPlaylistSongList()) {
      GuildSong gs = gpls.getGuildSong();
      sb.append(String.format(fmt, ++count, gs.getSong().getTitle(), gs.getSong().getArtist(), gs.getUuid()));
    }
    sb.append("```");
    event.getChannel().sendMessage("**" + "Current songs in playlists " + playlist + " for " + event.getMember().getAsMention() + "**").queue((succcess)->
    event.getChannel().sendMessage(sb.toString()).queue());
  }

  @Override
  public String help() {
    // TODO Auto-generated method stub
    return null;
  }

}
