package net.locusworks.discord.eighttrack.events.playlist.handlers;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.locusworks.discord.eighttrack.database.entities.GuildPlaylist;
import net.locusworks.discord.eighttrack.database.entities.GuildPlaylistSong;
import net.locusworks.discord.eighttrack.database.entities.Song;
import net.locusworks.discord.eighttrack.events.playlist.AbstractPlaylistEventHandler;
import net.locusworks.discord.eighttrack.handlers.GuildMusicHandler;
import net.locusworks.discord.eighttrack.services.RepositoryService;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

@Component
public class PlaylistPlayHandler extends AbstractPlaylistEventHandler {
  
  @Autowired
  private RepositoryService guildRepoService;
  
  private ApplicationLogger logger;
  public PlaylistPlayHandler() {
    super("play");
    logger = ApplicationLoggerFactory.getLogger(PlaylistPlayHandler.class);
  }

  @Override
  public void executeEvent(GuildMusicHandler musicHandler, GuildMessageReceivedEvent event, List<String> commands) {
    if (commands == null || commands.isEmpty()) {
      event.getChannel().sendMessage(event.getAuthor().getAsMention() + " please provide the name of the playlist to play").queue();
      return;
    }
    String playlist = commands.remove(0);
    
    
    GuildPlaylist currentPlaylist = guildRepoService.getGuildPlaylistRepo().findByGuildAndUserIdAndPlaylist(event.getGuild().getIdLong(), event.getMember().getIdLong(), playlist);
    
    if (currentPlaylist == null) {
      sendMessage(event, event.getAuthor().getAsMention() + ", no playlist with the name of " + playlist + " exists");
      return;
    }
    
    musicHandler.setCurrentPlaylist(currentPlaylist);

    musicHandler.getTrackManager().clearTracks();

    for(GuildPlaylistSong gpls : guildRepoService.getGuildPlaylistSongRepo().findByGuildPlaylist(currentPlaylist)) {
      Song s = gpls.getGuildSong().getSong();
      musicHandler.getTrackManager().queueLast(s.getFilePath());
    }

    try {
      musicHandler.isPlaying(false);
      musicHandler.play(event);
      musicHandler.isPlaying(true);
    } catch (Exception ex) {
      sendMessage(event, "Sorry " + event.getAuthor().getAsMention() + " I was unable to remove songs from the playlist. Reason: " + ex.getMessage(), true);
      logger.error("Unable to play playlist : " + ex.getMessage());
      logger.error(ex);
      guildRepoService.logEntry(event.getGuild().getIdLong(), ex, "Unable to play playlist: %s", ex.getMessage());
    }
  }

  @Override
  public String help() {
    // TODO Auto-generated method stub
    return null;
  }

}
